package service

import (
	"context"
	"encoding/json"
	"fmt"
	"io/ioutil"

	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
)

func (srv *serviceImpl) GoogleSignIn(
	ctx context.Context,
	req *oauth.GoogleSignInRequest,
) (*oauth.GoogleSignInResponse, error) {
	token, err := srv.config.OAuth.Google.Exchange(ctx, req.GetAuthorizationCode())
	if err != nil {
		return nil, err
	}

	client := srv.config.OAuth.Google.Client(ctx, token)
	url := fmt.Sprintf(srv.config.OAuth.Google.UserInfoURL, token.Extra("id_token"))

	resp, err := client.Get(url)
	if err != nil {
		return nil, err
	}

	defer func() {
		_ = resp.Body.Close()
	}()

	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}

	var data struct {
		Sub        string `json:"sub"`
		Email      string `json:"email"`
		GivenName  string `json:"given_name"`
		FamilyName string `json:"family_name"`
		Picture    string `json:"picture"`
	}

	err = json.Unmarshal(body, &data)
	if err != nil {
		return nil, err
	}

	return &oauth.GoogleSignInResponse{
		Success: true,
		UserInfo: &oauth.GoogleUserInfo{
			Id:         data.Sub,
			Email:      data.Email,
			GivenName:  data.GivenName,
			FamilyName: data.FamilyName,
			PictureUrl: data.Picture,
		},
	}, nil
}
