package test

import (
	"errors"
	"io/ioutil"
	"net/http"
	"strings"

	"github.com/golang/mock/gomock"
	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"golang.org/x/oauth2"
)

func (s *serviceSuite) TestFacebookGetUserInfoWithToken_ok() {
	req := &oauth.TokenAccessRequest{
		AccessToken: "aaa1",
	}

	tok := &oauth2.Token{
		AccessToken: req.GetAccessToken(),
		TokenType:   "bearer",
	}

	s.oauth2Provider.EXPECT().Client(s.testCtx, tok).Return(s.client)

	info := `{
  "id":         "ccc1",
  "email":      "ccc2",
  "first_name": "ccc3",
  "last_name":  "ccc4"
}
`

	s.roundTripper.EXPECT().RoundTrip(gomock.Any()).DoAndReturn(func(req *http.Request) (*http.Response, error) {
		return &http.Response{
			Body: ioutil.NopCloser(strings.NewReader(info)),
		}, nil
	})

	resp, err := s.srv.FacebookGetUserInfoWithToken(s.testCtx, req)
	s.Require().NoError(err)

	s.Require().Equal(&oauth.FacebookGetUserInfoResponse{
		Id:        "ccc1",
		Email:     "ccc2",
		FirstName: "ccc3",
		LastName:  "ccc4",
		Raw:       ([]byte)(info),
	}, resp)
}

func (s *serviceSuite) TestFacebookGetUserInfoWithToken_client() {
	req := &oauth.TokenAccessRequest{
		AccessToken: "aaa1",
	}

	tok := &oauth2.Token{
		AccessToken: req.GetAccessToken(),
		TokenType:   "bearer",
	}

	s.oauth2Provider.EXPECT().Client(s.testCtx, tok).Return(s.client)

	s.roundTripper.EXPECT().RoundTrip(gomock.Any()).DoAndReturn(func(req *http.Request) (*http.Response, error) {
		return nil, errors.New("error")
	})

	_, err := s.srv.FacebookGetUserInfoWithToken(s.testCtx, req)
	s.Require().Error(err)
}
