package test

import (
	"errors"
	"time"

	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/model"
	"golang.org/x/oauth2"
)

func (s *serviceSuite) TestInstagramExchangeToken_ok() {
	req := &oauth.InstagramExchangeTokenRequest{
		Ref:  "aaa1",
		Code: "aaa2",
	}

	s.oauth2Provider.EXPECT().Exchange(s.testCtx, req.GetCode()).Return(&oauth2.Token{
		AccessToken:  "bbb1",
		TokenType:    "bbb2",
		RefreshToken: "bbb3",
		Expiry:       time.Unix(100, 10),
	}, nil)

	s.repo.EXPECT().InstagramTokenSave(s.testCtx, &model.Oauth2Token{
		ID: req.GetRef(),
		Token: &oauth2.Token{
			AccessToken:  "bbb1",
			TokenType:    "bbb2",
			RefreshToken: "bbb3",
			Expiry:       time.Unix(100, 10),
		},
	}).Return(nil)

	resp, err := s.srv.InstagramExchangeToken(s.testCtx, req)
	s.Require().NoError(err)

	s.Require().Equal(&oauth.InstagramExchangeTokenResponse{}, resp)
}

func (s *serviceSuite) TestInstagramExchangeToken_error_save() {
	req := &oauth.InstagramExchangeTokenRequest{
		Ref:  "aaa1",
		Code: "aaa2",
	}

	s.oauth2Provider.EXPECT().Exchange(s.testCtx, req.GetCode()).Return(&oauth2.Token{
		AccessToken:  "bbb1",
		TokenType:    "bbb2",
		RefreshToken: "bbb3",
		Expiry:       time.Unix(100, 10),
	}, nil)

	s.repo.EXPECT().InstagramTokenSave(s.testCtx, &model.Oauth2Token{
		ID: req.GetRef(),
		Token: &oauth2.Token{
			AccessToken:  "bbb1",
			TokenType:    "bbb2",
			RefreshToken: "bbb3",
			Expiry:       time.Unix(100, 10),
		},
	}).Return(errors.New("error"))

	_, err := s.srv.InstagramExchangeToken(s.testCtx, req)
	s.Require().Error(err)
}

func (s *serviceSuite) TestInstagramExchangeToken_error_exchange() {
	req := &oauth.InstagramExchangeTokenRequest{
		Ref:  "aaa1",
		Code: "aaa2",
	}

	s.oauth2Provider.EXPECT().Exchange(s.testCtx, req.GetCode()).Return(nil, errors.New("error"))

	_, err := s.srv.InstagramExchangeToken(s.testCtx, req)
	s.Require().Error(err)
}
