package test

import (
	"errors"
	"io/ioutil"
	"net/http"
	"strings"

	"github.com/golang/mock/gomock"
	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"golang.org/x/oauth2"
)

func (s *serviceSuite) TestInstagramGetUserInfoWithToken_ok() {
	req := &oauth.TokenAccessRequest{
		AccessToken: "aaa1",
	}

	tok := &oauth2.Token{
		AccessToken: req.GetAccessToken(),
		TokenType:   "bearer",
	}

	s.oauth2Provider.EXPECT().Client(s.testCtx, tok).Return(s.client)

	info := `{
  "data": {
    "id":              "ccc1",
    "username":        "ccc2",
    "full_name":       "ccc3",
    "profile_picture": "ccc4",
    "bio":             "ccc5",
    "website":         "ccc6",
    "is_business":     false,
    "counts": {
      "media":       1,
      "follows":     2,
      "followed_by": 3
    }
  }
}
`

	s.roundTripper.EXPECT().RoundTrip(gomock.Any()).DoAndReturn(func(req *http.Request) (*http.Response, error) {
		return &http.Response{
			Body: ioutil.NopCloser(strings.NewReader(info)),
		}, nil
	})

	resp, err := s.srv.InstagramGetUserInfoWithToken(s.testCtx, req)
	s.Require().NoError(err)

	s.Require().Equal(&oauth.InstagramGetUserInfoResponse{
		Username:       "ccc2",
		FullName:       "ccc3",
		ProfilePicture: "ccc4",
		Raw:            ([]byte)(info),
	}, resp)
}

func (s *serviceSuite) TestInstagramGetUserInfoWithToken_client() {
	req := &oauth.TokenAccessRequest{
		AccessToken: "aaa1",
	}

	tok := &oauth2.Token{
		AccessToken: req.GetAccessToken(),
		TokenType:   "bearer",
	}

	s.oauth2Provider.EXPECT().Client(s.testCtx, tok).Return(s.client)

	s.roundTripper.EXPECT().RoundTrip(gomock.Any()).DoAndReturn(func(req *http.Request) (*http.Response, error) {
		return nil, errors.New("error")
	})

	_, err := s.srv.InstagramGetUserInfoWithToken(s.testCtx, req)
	s.Require().Error(err)
}
