package test

import (
	"context"
	"net/http"
	"testing"

	"github.com/golang/mock/gomock"
	"github.com/stretchr/testify/suite"
	repo "gitlab.mashroom.com/mashroom/backend/oauth/internal/model/mock"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/service"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/service/mock"
)

type serviceSuite struct {
	suite.Suite
	srv                    service.Service
	rootCtx, testCtx       context.Context
	rootCancel, testCancel context.CancelFunc
	ctrl                   *gomock.Controller
	repo                   *repo.MockRepository
	oauth2Provider         *mock.MockOAuth2Provider
	oauth1Provider         *mock.MockOAuth1Provider
	roundTripper           *mock.MockRoundTripper
	client                 *http.Client
}

func (s *serviceSuite) SetupSuite() {
	s.rootCtx, s.rootCancel = context.WithCancel(context.Background())
}

func (s *serviceSuite) SetupTest() {
	s.ctrl = gomock.NewController(s.T())
	s.repo = repo.NewMockRepository(s.ctrl)
	s.oauth2Provider = mock.NewMockOAuth2Provider(s.ctrl)
	s.oauth1Provider = mock.NewMockOAuth1Provider(s.ctrl)
	s.roundTripper = mock.NewMockRoundTripper(s.ctrl)
	s.client = &http.Client{
		Transport: s.roundTripper,
	}

	s.testCtx, s.testCancel = context.WithCancel(s.rootCtx)

	var err error

	s.srv, err = service.NewService(s.rootCtx, &service.Config{
		Repository: s.repo,
		OAuth: service.OAuth{
			Facebook: service.OAuth2Config{
				OAuth2Provider: s.oauth2Provider,
				UserInfoURL:    "http://example.com",
			},
			Instagram: service.OAuth2Config{
				OAuth2Provider: s.oauth2Provider,
				UserInfoURL:    "http://example.com?foo=%s",
			},
			Google: service.OAuth2Config{
				OAuth2Provider: s.oauth2Provider,
				UserInfoURL:    "http://example.com?foo=%s",
			},
			Twitter: service.OAuth1Config{
				OAuth1Provider: s.oauth1Provider,
			},
		},
	})
	s.Require().NoError(err)
}

func (s *serviceSuite) TearDownTest() {
	s.ctrl.Finish()

	s.testCancel()
}

func (s *serviceSuite) TearSuite() {
	s.rootCancel()
}

func TestService(t *testing.T) {
	suite.Run(t, &serviceSuite{})
}
