package test

import (
	"errors"
	"net/url"

	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/model"
)

func (s *serviceSuite) TestTwitterAuthURL_ok() {
	req := &oauth.TwitterAuthUrlRequest{
		Ref: "aaa1",
	}

	reqp := &model.TwitterRequestPair{
		ID:            req.GetRef(),
		RequestToken:  "rrr1",
		RequestSecret: "rrr2",
	}

	s.oauth1Provider.EXPECT().RequestToken().Return(reqp.RequestToken, reqp.RequestSecret, nil)

	s.repo.EXPECT().TwitterTokenRequestSave(s.testCtx, reqp).Return(nil)

	uuu, _ := url.Parse("http://example.com")

	s.oauth1Provider.EXPECT().AuthorizationURL(reqp.RequestToken).Return(uuu, nil)

	resp, err := s.srv.TwitterAuthUrl(s.testCtx, req)
	s.Require().NoError(err)

	s.Require().Equal(&oauth.TwitterAuthUrlResponse{
		Url: uuu.String(),
	}, resp)
}

func (s *serviceSuite) TestTwitterAuthURL_error_url() {
	req := &oauth.TwitterAuthUrlRequest{
		Ref: "aaa1",
	}

	reqp := &model.TwitterRequestPair{
		ID:            req.GetRef(),
		RequestToken:  "rrr1",
		RequestSecret: "rrr2",
	}

	s.oauth1Provider.EXPECT().RequestToken().Return(reqp.RequestToken, reqp.RequestSecret, nil)

	s.repo.EXPECT().TwitterTokenRequestSave(s.testCtx, reqp).Return(nil)

	s.oauth1Provider.EXPECT().AuthorizationURL(reqp.RequestToken).Return(nil, errors.New("error"))

	_, err := s.srv.TwitterAuthUrl(s.testCtx, req)
	s.Require().Error(err)
}

func (s *serviceSuite) TestTwitterAuthURL_error_save() {
	req := &oauth.TwitterAuthUrlRequest{
		Ref: "aaa1",
	}

	reqp := &model.TwitterRequestPair{
		ID:            req.GetRef(),
		RequestToken:  "rrr1",
		RequestSecret: "rrr2",
	}

	s.oauth1Provider.EXPECT().RequestToken().Return(reqp.RequestToken, reqp.RequestSecret, nil)

	s.repo.EXPECT().TwitterTokenRequestSave(s.testCtx, reqp).Return(errors.New("error"))

	_, err := s.srv.TwitterAuthUrl(s.testCtx, req)
	s.Require().Error(err)
}

func (s *serviceSuite) TestTwitterAuthURL_error_request_token() {
	req := &oauth.TwitterAuthUrlRequest{
		Ref: "aaa1",
	}

	s.oauth1Provider.EXPECT().RequestToken().Return("", "", errors.New("error"))

	_, err := s.srv.TwitterAuthUrl(s.testCtx, req)
	s.Require().Error(err)
}
