package test

import (
	"errors"

	"github.com/dghubble/oauth1"
	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/model"
)

func (s *serviceSuite) TestTwitterExchangeToken_ok() {
	req := &oauth.TwitterExchangeTokenRequest{
		Ref:      "aaa1",
		Verifier: "aaa2",
	}

	reqp := &model.TwitterRequestPair{
		ID:            req.GetRef(),
		RequestToken:  "rrr1",
		RequestSecret: "rrr2",
	}

	s.repo.EXPECT().TwitterTokenRequestGetByID(s.testCtx, req.GetRef()).Return(reqp, nil)

	tok := &oauth1.Token{
		Token:       "acc1",
		TokenSecret: "acc2",
	}

	s.oauth1Provider.EXPECT().AccessToken(reqp.RequestToken, reqp.RequestSecret, req.GetVerifier()).Return(
		tok.Token,
		tok.TokenSecret,
		nil,
	)

	s.repo.EXPECT().TwitterTokenSave(s.testCtx, &model.TwitterToken{
		ID:    req.GetRef(),
		Token: tok,
	}).Return(nil)

	resp, err := s.srv.TwitterExchangeToken(s.testCtx, req)
	s.Require().NoError(err)

	s.Require().Equal(&oauth.TwitterExchangeTokenResponse{}, resp)
}

func (s *serviceSuite) TestTwitterExchangeToken_error_save() {
	req := &oauth.TwitterExchangeTokenRequest{
		Ref:      "aaa1",
		Verifier: "aaa2",
	}

	reqp := &model.TwitterRequestPair{
		ID:            req.GetRef(),
		RequestToken:  "rrr1",
		RequestSecret: "rrr2",
	}

	s.repo.EXPECT().TwitterTokenRequestGetByID(s.testCtx, req.GetRef()).Return(reqp, nil)

	tok := &oauth1.Token{
		Token:       "acc1",
		TokenSecret: "acc2",
	}

	s.oauth1Provider.EXPECT().AccessToken(reqp.RequestToken, reqp.RequestSecret, req.GetVerifier()).Return(
		tok.Token,
		tok.TokenSecret,
		nil,
	)

	s.repo.EXPECT().TwitterTokenSave(s.testCtx, &model.TwitterToken{
		ID:    req.GetRef(),
		Token: tok,
	}).Return(errors.New("error"))

	_, err := s.srv.TwitterExchangeToken(s.testCtx, req)
	s.Require().Error(err)
}

func (s *serviceSuite) TestTwitterExchangeToken_error_access() {
	req := &oauth.TwitterExchangeTokenRequest{
		Ref:      "aaa1",
		Verifier: "aaa2",
	}

	reqp := &model.TwitterRequestPair{
		ID:            req.GetRef(),
		RequestToken:  "rrr1",
		RequestSecret: "rrr2",
	}

	s.repo.EXPECT().TwitterTokenRequestGetByID(s.testCtx, req.GetRef()).Return(reqp, nil)

	s.oauth1Provider.EXPECT().AccessToken(reqp.RequestToken, reqp.RequestSecret, req.GetVerifier()).Return(
		"",
		"",
		errors.New("error"),
	)

	_, err := s.srv.TwitterExchangeToken(s.testCtx, req)
	s.Require().Error(err)
}
