package test

import (
	"errors"
	"io/ioutil"
	"net/http"
	"strings"

	"github.com/dghubble/oauth1"
	"github.com/golang/mock/gomock"
	oauth "gitlab.mashroom.com/mashroom/backend/oauth/api"
	"gitlab.mashroom.com/mashroom/backend/oauth/internal/model"
)

func (s *serviceSuite) TestTwitterStatusUpdate_ok() {
	req := &oauth.TwitterStatusUpdateRequest{
		Ref:    "aaa1",
		Status: "aaa2",
	}

	tok := &model.TwitterToken{
		ID: req.GetRef(),
		Token: &oauth1.Token{
			Token:       "rrr1",
			TokenSecret: "rrr2",
		},
	}

	s.repo.EXPECT().TwitterTokenGetByID(s.testCtx, req.GetRef()).Return(tok, nil)

	s.oauth1Provider.EXPECT().Client(s.testCtx, tok.Token).Return(s.client)

	info := `{}`

	s.roundTripper.EXPECT().RoundTrip(gomock.Any()).DoAndReturn(func(req *http.Request) (*http.Response, error) {
		return &http.Response{
			StatusCode: http.StatusOK,
			Status:     http.StatusText(http.StatusOK),
			Body:       ioutil.NopCloser(strings.NewReader(info)),
		}, nil
	})

	resp, err := s.srv.TwitterStatusUpdate(s.testCtx, req)
	s.Require().NoError(err)

	s.Require().Equal(&oauth.TwitterStatusUpdateResponse{}, resp)
}

func (s *serviceSuite) TestTwitterStatusUpdate_error_client() {
	req := &oauth.TwitterStatusUpdateRequest{
		Ref:    "aaa1",
		Status: "aaa2",
	}

	tok := &model.TwitterToken{
		ID: req.GetRef(),
		Token: &oauth1.Token{
			Token:       "rrr1",
			TokenSecret: "rrr2",
		},
	}

	s.repo.EXPECT().TwitterTokenGetByID(s.testCtx, req.GetRef()).Return(tok, nil)

	s.oauth1Provider.EXPECT().Client(s.testCtx, tok.Token).Return(s.client)

	s.roundTripper.EXPECT().RoundTrip(gomock.Any()).DoAndReturn(func(req *http.Request) (*http.Response, error) {
		return nil, errors.New("error")
	})

	_, err := s.srv.TwitterStatusUpdate(s.testCtx, req)
	s.Require().Error(err)
}

func (s *serviceSuite) TestTwitterStatusUpdate_error_get() {
	req := &oauth.TwitterStatusUpdateRequest{
		Ref:    "aaa1",
		Status: "aaa2",
	}

	s.repo.EXPECT().TwitterTokenGetByID(s.testCtx, req.GetRef()).Return(nil, errors.New("error"))

	_, err := s.srv.TwitterStatusUpdate(s.testCtx, req)
	s.Require().Error(err)
}
